import unittest2 as unittest
from mock import patch

from pyopencell.resources.customer import Customer


from .settings import REQUIRED_ENVVARS


class CustomerTests(unittest.TestCase):

    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.customer.Client.get")
    def test_get(self, client_get_mock):
        expected_code = "some-customer-code"
        expected_response_data = {
            "customer": {
                "code": expected_code
            },
            "actionStatus": {
                "status": "SUCCESS",
                "errorCode": "",
                "message": "",
            }
        }
        client_get_mock.return_value = expected_response_data

        response = Customer.get(expected_code)

        for key, value in expected_response_data["customer"].items():
            self.assertEquals(getattr(response.customer, key), value)

        for key, value in expected_response_data["actionStatus"].items():
            self.assertEquals(getattr(response.action_status, key), value)

        client_get_mock.assert_called_with(Customer._url_path, customerCode=expected_code)

    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.customer.Client.post")
    def test_create(self, client_post_mock):
        expected_response_data = {
            "status": "SUCCESS",
            "errorCode": "",
            "message": "",
        }
        client_post_mock.return_value = expected_response_data

        expected_code = "some-customer-code"

        customer_data = {
            "code": expected_code,
            "name": {
                "title": None,
                "firstName": "Nombre",
                "lastName": "Apellido"
            },
        }

        response = Customer.create(**customer_data)

        for key, value in expected_response_data.items():
            self.assertEquals(getattr(response, key), value)

        client_post_mock.assert_called_with(Customer._url_path, customer_data)

    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.customer.Client.put")
    def test_update(self, client_put_mock):
        expected_response_data = {
            "status": "SUCCESS",
            "errorCode": "",
            "message": "",
        }
        client_put_mock.return_value = expected_response_data

        expected_code = "some-customer-code"

        customer_data = {
            "code": expected_code,
            "name": {
                "title": None,
                "firstName": "Nombre",
                "lastName": "Apellido"
            },
        }

        response = Customer.update(**customer_data)

        for key, value in expected_response_data.items():
            self.assertEquals(getattr(response, key), value)

        client_put_mock.assert_called_with(Customer._url_path, customer_data)



import unittest
from pyopencell import helpers


class HelpersTest(unittest.TestCase):
    def test_getenv_fails_if_not_defined_envvar(self):
        with self.assertRaises(Exception):
            helpers.getenv_or_fail('Fooo')



import unittest2 as unittest
from mock import patch

from pyopencell.resources.crm_account_hierarchy import CRMAccountHierarchy

from .settings import REQUIRED_ENVVARS


class CRMAccountHierarchyTests(unittest.TestCase):

    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.crm_account_hierarchy.Client.post")
    def test_create(self, client_post_mock):
        expected_response_data = {
            "status": "SUCCESS",
            "errorCode": "",
            "message": "",
        }
        client_post_mock.return_value = expected_response_data

        attributes = {
            "code": "hola"
        }

        response = CRMAccountHierarchy.create(**attributes)
        for key, value in expected_response_data.items():
            self.assertEquals(getattr(response, key), value)

        client_post_mock.assert_called_with(
            "{}/{}".format(CRMAccountHierarchy._url_path, 'createCRMAccountHierarchy'),
            body=attributes
        )

    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.crm_account_hierarchy.Client.post")
    def test_update(self, client_post_mock):
        expected_response_data = {
            "status": "SUCCESS",
            "errorCode": "",
            "message": "",
        }
        client_post_mock.return_value = expected_response_data

        attributes = {
            "code": "hola"
        }

        response = CRMAccountHierarchy.update(**attributes)
        for key, value in expected_response_data.items():
            self.assertEquals(getattr(response, key), value)

        client_post_mock.assert_called_with(
            "{}/{}".format(CRMAccountHierarchy._url_path, 'createOrUpdateCRMAccountHierarchy'),
            body=attributes
        )



# -*- coding: utf-8 -*-
import unittest2 as unittest
from mock import patch

from pyopencell.resources.customer_account import CustomerAccount

from .settings import REQUIRED_ENVVARS


class CustomerAccountTests(unittest.TestCase):
    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.customer_account.Client.get")
    def test_get(self, client_get_mock):
        expected_code = "some-customer-code"
        expected_response_data = {
            "customerAccount": {
                "code": expected_code
            },
            "actionStatus": {
                "status": "SUCCESS",
                "errorCode": "",
                "message": "",
            }
        }
        client_get_mock.return_value = expected_response_data

        response = CustomerAccount.get(expected_code)

        for key, value in expected_response_data["customerAccount"].items():
            self.assertEquals(getattr(response.customerAccount, key), value)

        for key, value in expected_response_data["actionStatus"].items():
            self.assertEquals(getattr(response.action_status, key), value)

        client_get_mock.assert_called_with(
            CustomerAccount._url_path,
            customerAccountCode=expected_code
        )



import unittest2 as unittest
from mock import patch

from pyopencell.resources.subscription import Subscription
from pyopencell.responses.action_status import ActionStatus

from .settings import REQUIRED_ENVVARS


@patch.dict('os.environ', REQUIRED_ENVVARS)
class SubscriptionTests(unittest.TestCase):

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get_not_subscription_returned(self, client_get_mock):
        expected_code = "some-sub-code"
        expected_response_data = {
            "status": "FAIL",
            "errorCode": "",
            "message": "",
        }
        client_get_mock.return_value = expected_response_data

        response = Subscription.get(expected_code)

        client_get_mock.assert_called_with(Subscription._url_path, subscriptionCode=expected_code)
        self.assertIsInstance(response, ActionStatus)

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get(self, client_get_mock):
        expected_code = "some-sub-code"
        expected_response_data = {
            "subscription": {
                "code": expected_code
            },
            "actionStatus": {
                "status": "SUCCESS",
                "errorCode": "",
                "message": "",
            }
        }
        client_get_mock.return_value = expected_response_data

        response = Subscription.get(expected_code)

        for key, value in expected_response_data["subscription"].items():
            self.assertEquals(getattr(response.subscription, key), value)

        for key, value in expected_response_data["actionStatus"].items():
            self.assertEquals(getattr(response.action_status, key), value)

        client_get_mock.assert_called_with(Subscription._url_path, subscriptionCode=expected_code)

    @patch("pyopencell.resources.subscription.Client.post")
    def test_create(self, client_post_mock):
        attributes = {
            "code": "hola"
        }

        Subscription.create(**attributes)

        client_post_mock.assert_called_with(
            Subscription._url_path,
            attributes
        )

    @patch("pyopencell.resources.subscription.Client.post")
    def test_activate(self, client_post_mock):
        services_to_activate = [
            {
                "service": "foO",
                "quantity": 1,
                "subscriptionDate": "XX/XX/XXXX"
            }
        ]

        subscription = Subscription()
        subscription.code = 1
        subscription.activate(services_to_activate)

        client_post_mock.assert_called_with(
            "{}/{}".format(subscription._url_path, "activateServices"),
            {
                "subscription": 1,
                "servicesToActivate": {
                    "service": services_to_activate
                }
            }
        )

    @patch("pyopencell.resources.subscription.Client.post")
    def test_terminate(self, client_post_mock):
        termination_date = "2019-01-25"

        subscription = Subscription()
        subscription.code = 1
        subscription.terminate(termination_date)

        client_post_mock.assert_called_with(
            "{}/{}".format(subscription._url_path, "terminate"),
            {
                "subscriptionCode": subscription.code,
                "terminationDate": termination_date,
                "terminationReason": "CC_TERMINATION"
            }
        )

    @patch("pyopencell.resources.subscription.Client.post")
    def test_terminate_services(self, client_post_mock):
        termination_date = "2019-01-25"
        services = ["service-code-1", "service-code-2"]

        subscription = Subscription()
        subscription.code = 1
        subscription.terminateServices(termination_date, services)

        client_post_mock.assert_called_with(
            "{}/{}".format(subscription._url_path, "terminateServices"),
            {
                "subscriptionCode": subscription.code,
                "terminationDate": termination_date,
                "terminationReason": "CC_TERMINATION",
                "services": services,
            }
        )

    @patch("pyopencell.resources.subscription.Client.post")
    def test_apply_one_shot_charge(self, client_post_mock):
        one_shot_charge_code = "FoOo"

        subscription = Subscription()
        subscription.code = 1
        subscription.applyOneShotCharge(one_shot_charge_code)

        client_post_mock.assert_called_with(
            "{}/{}".format(subscription._url_path, "applyOneShotChargeInstance"),
            {
                "subscription": subscription.code,
                "oneShotCharge": one_shot_charge_code,
            }
        )

    @patch("pyopencell.resources.subscription.Client.post")
    def test_apply_one_shot_charge_with_amount(self, client_post_mock):
        one_shot_charge_code = "FoOo"
        amountWithoutTax = -123.4567

        subscription = Subscription()
        subscription.code = 1
        subscription.applyOneShotCharge(one_shot_charge_code, amountWithoutTax)

        client_post_mock.assert_called_with(
            "{}/{}".format(subscription._url_path, "applyOneShotChargeInstance"),
            {
                "subscription": subscription.code,
                "oneShotCharge": one_shot_charge_code,
                "amountWithoutTax": amountWithoutTax
            }
        )



from datetime import datetime
from mock import patch
import unittest2 as unittest

from pyopencell.resources.invoice_list import InvoiceList
from pyopencell.resources.invoice import Invoice
from pyopencell.responses.action_status import ActionStatus

from .settings import REQUIRED_ENVVARS


@patch.dict('os.environ', REQUIRED_ENVVARS)
class InvoiceListTests(unittest.TestCase):

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get_status_not_success(self, client_get_mock):
        expected_response_data = {
            "status": "FAIL",
            "errorCode": "",
            "message": ""
        }

        client_get_mock.return_value = expected_response_data

        response = InvoiceList.get()

        self.assertIsInstance(response, ActionStatus)

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get(self, client_get_mock):
        expected_response_data = {
            "invoices": [
                {
                    "invoiceId": 12345,
                    "amountTax": 12345.0,
                    "categoryInvoiceAgregate": [
                        {
                            "amountTax": 12345.0,
                        }
                    ],
                    "taxAggregate": [
                        {
                            "amountTax": 12345.0,
                        }
                    ],
                }
            ],
            "actionStatus": {
                "status": "SUCCESS",
                "errorCode": "",
                "message": "",
            },
            "paging": {
                "fullTextFilter": "...",
                "filters": {
                    "property1": {},
                    "property2": {}
                },
                "fields": "...",
                "offset": 12345,
                "limit": 12345,
                "sortBy": "...",
                "sortOrder": "ASCENDING",
                "totalNumberOfRecords": 12345,
                "loadReferenceDepth": 12345
            }
        }
        client_get_mock.return_value = expected_response_data

        response = InvoiceList.get()

        for key, value in expected_response_data["actionStatus"].items():
            self.assertEquals(getattr(response.action_status, key), value)

        for key, value in expected_response_data["paging"].items():
            self.assertEquals(getattr(response.paging, key), value)

        for invoice in response.invoices:
            self.assertIsInstance(invoice, Invoice)

        client_get_mock.assert_called_with("/invoice/list", query=None, limit=10, offset=None)

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get_filtering_by_date(self, client_get_mock):
        now = datetime.now()
        InvoiceList.filter_by_date(now)

        client_get_mock.assert_called_with(
            "/invoice/list",
            query="invoiceDate:{}".format(now.strftime("%Y-%m-%d")),
            limit=10,
            offset=None)

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get_with_offset(self, client_get_mock):
        InvoiceList.get(limit=10, offset=2)

        client_get_mock.assert_called_with("/invoice/list", query=None, limit=10, offset=2)



from __future__ import unicode_literals  # support both Python2 and 3

from mock import patch
import unittest2 as unittest

import json
import os

from pyopencell.client import Client
from pyopencell import exceptions

from .settings import REQUIRED_ENVVARS


class FakeRequest:
    method = ""
    url = ""

    def __init__(self, method="GET", url="some-url"):
        self.method = method
        self.url = url


class FakeResponse:
    status_code = None
    content = ""
    request = FakeRequest()

    def __init__(self, status=200, content="{}"):
        self.status_code = status
        self.content = content

    def json(self):
        return json.loads(self.content)


@patch.dict('os.environ', REQUIRED_ENVVARS)
class ClientTests(unittest.TestCase):
    expected_headers = {
        'Content-Type': 'application/json',
        'Accept': 'application/json'
    }
    sample_body = {'a': 1}
    sample_route = '/path'

    def test_init_has_not_envvars_defined_raises_exception(self):
        with self.assertRaises(Exception):
            for envvar in self.required_envvars.keys():
                os.unsetenv(envvar)
                Client()

    @patch('pyopencell.client.requests.request', side_effect=Exception)
    def test_network_error_raises_expected_exception(self, _):
        with self.assertRaises(exceptions.PyOpenCellHTTPException):
            Client().get(self.sample_route)

    @patch('pyopencell.client.requests.request', return_value=FakeResponse(status=500))
    def test_server_error_500_raises_expected_exception(self, request_mock):
        request_mock.return_value = FakeResponse(status=500)
        with self.assertRaises(exceptions.PyOpenCellAPIException):
            Client().get(self.sample_route)

    @patch('pyopencell.client.requests.request', return_value=FakeResponse(status=404, content=""))
    def test_server_response_40x_with_empty_body_raises_expected_exception(self, request_mock):
        request_mock.return_value = FakeResponse(status=404, content="")
        with self.assertRaises(exceptions.PyOpenCellAPIException):
            Client().get(self.sample_route)

    @patch('pyopencell.client.requests.request')
    def test_server_response_40x_raises_expected_exception(self, request_mock):
        expected_response = FakeResponse(status=400)

        request_mock.return_value = expected_response

        with self.assertRaises(exceptions.PyOpenCellAPIException):
            Client().post(self.sample_route, self.sample_body)

    @patch('pyopencell.client.requests.request')
    def test_server_response_20x_with_errorCode_raise_expected_exception(self, request_mock):
        request_mock.return_value = FakeResponse(content='{"errorCode": "Error XXX"}')
        with self.assertRaises(exceptions.PyOpenCellAPIException):
            Client().get(self.sample_route)

    @patch('pyopencell.client.requests.request')
    def test_server_response_20x_with_status_FAIL_raise_expected_exception(self, request_mock):
        request_mock.return_value = FakeResponse(content='{"status": "FAIL"}')
        with self.assertRaises(exceptions.PyOpenCellAPIException):
            Client().get(self.sample_route)

    @patch('pyopencell.client.requests.request', return_value=FakeResponse())
    def test_get(self, mock_request):
        Client().get(self.sample_route)

        mock_request.assert_called_once_with(
            'GET',
            'http://myoc/api/rest/path',
            auth=(u'user', u'pwd'),
            data=None,
            params={},
            headers=self.expected_headers,
        )

    @patch('pyopencell.client.requests.request', return_value=FakeResponse())
    def test_post(self, mock_request):
        Client().post(self.sample_route, self.sample_body)

        mock_request.assert_called_once_with(
            'POST',
            'http://myoc/api/rest/path',
            auth=(u'user', u'pwd'),
            data=json.dumps(self.sample_body),
            params={},
            headers=self.expected_headers)

    @patch('pyopencell.client.requests.request', return_value=FakeResponse())
    def test_put(self, mock_request):
        Client().put(self.sample_route, self.sample_body)

        mock_request.assert_called_once_with(
            'PUT',
            'http://myoc/api/rest/path',
            auth=(u'user', u'pwd'),
            data=json.dumps(self.sample_body),
            params={},
            headers=self.expected_headers)

    @patch('pyopencell.client.requests.request', return_value=FakeResponse())
    def test_delete(self, mock_request):
        Client().delete(self.sample_route)

        mock_request.assert_called_once_with(
            'DELETE',
            'http://myoc/api/rest/path',
            auth=(u'user', u'pwd'),
            data=None,
            headers=self.expected_headers,
            params={})



from mock import patch
import unittest2 as unittest

from pyopencell.resources.invoice import Invoice
from pyopencell.responses.action_status import ActionStatus

from .settings import REQUIRED_ENVVARS


@patch.dict('os.environ', REQUIRED_ENVVARS)
@patch("pyopencell.resources.subscription.Client.post")
class InvoiceTests(unittest.TestCase):

    def test_sendByEmail_response(self, client_get_mock):
        expected_response_data = {
            "status": "SUCCESS",
            "errorCode": "",
            "message": ""
        }

        client_get_mock.return_value = expected_response_data

        response = Invoice.sendByEmail(0)

        self.assertIsInstance(response, ActionStatus)

    def test_sendByEmail_send_correct_data(self, client_post_mock):
        expected_response_data = {
            "status": "SUCCESS",
            "errorCode": "",
            "message": ""
        }
        invoice_id = 8

        client_post_mock.return_value = expected_response_data

        Invoice.sendByEmail(invoice_id)

        client_post_mock.assert_called_with(
            "{}/{}".format(Invoice._url_path, "sendByEmail"),
            {"invoiceId": invoice_id})



from mock import patch
import unittest2 as unittest

from pyopencell.resources.subscription_list import SubscriptionList
from pyopencell.resources.subscription import Subscription
from pyopencell.responses.action_status import ActionStatus

from .settings import REQUIRED_ENVVARS


@patch.dict("os.environ", REQUIRED_ENVVARS)
class SubscriptionListTests(unittest.TestCase):
    @patch("pyopencell.resources.subscription.Client.get")
    def test_get_status_not_success(self, client_get_mock):
        expected_response_data = {"status": "FAIL", "errorCode": "", "message": ""}

        client_get_mock.return_value = expected_response_data

        response = SubscriptionList.get()

        self.assertIsInstance(response, ActionStatus)

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get(self, client_get_mock):
        expected_response_data = {
            "paging": {
                "fullTextFilter": "...",
                "filters": {"property1": {}, "property2": {}},
                "fields": "...",
                "offset": 12345,
                "limit": 12345,
                "sortBy": "...",
                "sortOrder": "ASCENDING",
                "totalNumberOfRecords": 12345,
                "loadReferenceDepth": 12345,
            },
            "subscriptions": {
                "listSize": 12345,
                "subscription": [
                    {
                        "id": 12345,
                        "code": "...",
                        "auditable": {},
                        "auditableField": [{}, {}],
                        "description": "...",
                        "updatedCode": "...",
                        "userAccount": "...",
                        "offerTemplate": "...",
                        "subscriptionDate": 12345,
                        "terminationDate": 12345,
                        "endAgreementDate": 12345,
                        "status": "CREATED",
                        "statusDate": 12345,
                        "customFields": {},
                        "accesses": {},
                        "services": {},
                        "products": {},
                        "productInstance": [{}, {}],
                        "terminationReason": "...",
                        "orderNumber": "...",
                        "minimumAmountEl": "...",
                        "minimumAmountElSpark": "...",
                        "minimumLabelEl": "...",
                        "minimumLabelElSpark": "...",
                        "subscribedTillDate": 12345,
                        "renewed": True,
                        "renewalNotifiedDate": 12345,
                        "renewalRule": {},
                        "billingCycle": "...",
                        "seller": "...",
                        "autoEndOfEngagement": True,
                        "ratingGroup": "...",
                        "electronicBilling": True,
                        "email": "...",
                        "mailingType": "...",
                        "emailTemplate": "...",
                        "ccedEmails": "...",
                        "discountPlanForInstantiation": [{}, {}],
                        "discountPlanForTermination": ["...", "..."],
                        "discountPlanInstance": [{}, {}],
                    },
                    {
                        "id": 12345,
                        "code": "...",
                        "auditable": {},
                        "auditableField": [{}, {}],
                        "description": "...",
                        "updatedCode": "...",
                        "userAccount": "...",
                        "offerTemplate": "...",
                        "subscriptionDate": 12345,
                        "terminationDate": 12345,
                        "endAgreementDate": 12345,
                        "status": "ACTIVE",
                        "statusDate": 12345,
                        "customFields": {},
                        "accesses": {},
                        "services": {},
                        "products": {},
                        "productInstance": [{}, {}],
                        "terminationReason": "...",
                        "orderNumber": "...",
                        "minimumAmountEl": "...",
                        "minimumAmountElSpark": "...",
                        "minimumLabelEl": "...",
                        "minimumLabelElSpark": "...",
                        "subscribedTillDate": 12345,
                        "renewed": True,
                        "renewalNotifiedDate": 12345,
                        "renewalRule": {},
                        "billingCycle": "...",
                        "seller": "...",
                        "autoEndOfEngagement": True,
                        "ratingGroup": "...",
                        "electronicBilling": True,
                        "email": "...",
                        "mailingType": "...",
                        "emailTemplate": "...",
                        "ccedEmails": "...",
                        "discountPlanForInstantiation": [{}, {}],
                        "discountPlanForTermination": ["...", "..."],
                        "discountPlanInstance": [{}, {}],
                    },
                ],
            },
            "actionStatus": {
                "status": "SUCCESS",
            },
        }
        client_get_mock.return_value = expected_response_data

        response = SubscriptionList.get()

        for key, value in expected_response_data["actionStatus"].items():
            self.assertEquals(getattr(response.action_status, key), value)

        for key, value in expected_response_data["paging"].items():
            self.assertEquals(getattr(response.paging, key), value)

        for subscription in response.subscriptions:
            self.assertIsInstance(subscription, Subscription)

        client_get_mock.assert_called_with(
            "/billing/subscription/list", query=None, limit=10, offset=None
        )

    @patch("pyopencell.resources.subscription.Client.get")
    def test_get_with_offset(self, client_get_mock):
        SubscriptionList.get(limit=10, offset=2)

        client_get_mock.assert_called_with(
            "/billing/subscription/list", query=None, limit=10, offset=2
        )

    @patch("pyopencell.resources.subscription.Client.get")
    def test_query_without_matches(self, client_get_mock):
        # NOTE opencell API should return an empty array in
        # subscriptions.subscription, but life is hard!
        expected_response_data = {
            "actionStatus": {"status": "SUCCESS"},
            "paging": {
                "fullTextFilter": None,
                "filters": {"userAccount.code": "caca"},
                "fields": None,
                "offset": None,
                "limit": 10,
                "sortBy": "code",
                "sortOrder": "ASCENDING",
                "totalNumberOfRecords": 0,
                "loadReferenceDepth": 0,
            },
        }

        client_get_mock.return_value = expected_response_data

        subscription_list = SubscriptionList.get(query="userAccount.code:caca")

        self.assertEquals(len(subscription_list.subscriptions), 0)



import unittest2 as unittest
from mock import patch

from pyopencell.resources.access import Access

from .settings import REQUIRED_ENVVARS


class AccessTests(unittest.TestCase):

    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.customer.Client.get")
    def test_get(self, client_get_mock):
        expected_code = "some-access-code"
        expected_subscription_code = "some-subscription-code"

        Access.get(
            accessCode=expected_code,
            subscriptionCode=expected_subscription_code)

        client_get_mock.assert_called_with(
            Access._url_path,
            accessCode=expected_code,
            subscriptionCode=expected_subscription_code)

    @patch.dict('os.environ', REQUIRED_ENVVARS)
    @patch("pyopencell.resources.access.Client.post")
    def test_create(self, client_post_mock):
        expected_code = "some-access-code"

        access_data = {
            "code": expected_code,
            "subscriotion": "subs1",
            "startDate": None,
            "endDate": None,
            "disabled": False
        }

        Access.create(**access_data)

        client_post_mock.assert_called_with(Access._url_path, access_data)



import unittest
from pyopencell.responses.paged_response import PagedResponse
from pyopencell.resources.base_resource_list import BaseResourceList
from pyopencell.resources.base_resource import BaseResource


class Banana(BaseResource):
    _name = "banana"


class BananaList(BaseResourceList):
    items_resource_class = Banana
    _name = "bananas"


class NestedBananaList(BaseResourceList):
    items_resource_class = Banana
    _name = "bananas"

    @classmethod
    def get_instances_from_response(self, **response):
        return response['banana_tree']['bananas']


class PagedResponseTest(unittest.TestCase):
    def test_with_name_attribute(self):
        response = {
            "paging": {},
            "actionStatus": {},
            "bananas": [
                {"name": "one banana"},
                {"name": "two bananas"},
                {"name": "three bananas"}
            ]
        }

        paged_response = PagedResponse(BananaList, **response)

        self.assertEquals(
            [banana.name for banana in paged_response.bananas],
            [banana["name"] for banana in response["bananas"]]
        )

    def test_with_instances_from_response_method(self):
        response = {
            "paging": {},
            "actionStatus": {},
            "banana_tree": {
                "bananas": [
                    {"name": "one banana"},
                    {"name": "two bananas"},
                    {"name": "three bananas"}
                ]
            }
        }

        paged_response = PagedResponse(NestedBananaList, **response)

        self.assertEquals(
            [banana.name for banana in paged_response.bananas],
            [banana["name"] for banana in response["banana_tree"]["bananas"]]
        )
